@@ -41,6 +41,7 @@ import java.security.NoSuchAlgorithmException;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
 import java.util.Optional;
 import java.util.TreeMap;
@@ -594,10 +595,10 @@ public final class SAMUtils {
      */
     public static void makeReadUnmappedWithOriginalTags(final SAMRecord rec) {
         if (!hasOriginalMappingInformation(rec)) {
-            rec.setAttribute(SAMTag.OP.name(), rec.getAlignmentStart());
-            rec.setAttribute(SAMTag.OC.name(), rec.getCigarString());
-            rec.setAttribute(SAMTag.OF.name(), rec.getFlags());
-            rec.setAttribute(SAMTag.OR.name(), rec.getReferenceName());
+            rec.setAttribute(SAMTag.OP, rec.getAlignmentStart());
+            rec.setAttribute(SAMTag.OC, rec.getCigarString());
+            rec.setAttribute(SAMTag.OF, rec.getFlags());
+            rec.setAttribute(SAMTag.OR, rec.getReferenceName());
         }
         makeReadUnmapped(rec);
     }
@@ -606,10 +607,10 @@ public final class SAMUtils {
      * See if any tags pertaining to original mapping information have been set.
      */
     public static boolean hasOriginalMappingInformation(final SAMRecord rec) {
-        return rec.getAttribute(SAMTag.OP.name()) != null
-                || rec.getAttribute(SAMTag.OC.name()) != null
-                || rec.getAttribute(SAMTag.OF.name()) != null
-                || rec.getAttribute(SAMTag.OR.name()) != null;
+        return rec.getAttribute(SAMTag.OP) != null
+                || rec.getAttribute(SAMTag.OC) != null
+                || rec.getAttribute(SAMTag.OF) != null
+                || rec.getAttribute(SAMTag.OR) != null;
     }
 
     /**
@@ -808,7 +809,7 @@ public final class SAMUtils {
      * @return Mate Cigar String, or null if there is none.
      */
     public static String getMateCigarString(final SAMRecord rec) {
-        return rec.getStringAttribute(SAMTag.MC.name());
+        return rec.getStringAttribute(SAMTag.MC);
     }
 
     /**
@@ -1142,7 +1143,7 @@ public final class SAMUtils {
         if (record == null) throw new IllegalArgumentException("record is null");
         if (record.getHeader() == null) throw new IllegalArgumentException("record.getHeader() is null");
         /* extract value of SA tag */
-        final Object saValue = record.getAttribute(SAMTag.SA.getBinaryTag());
+        final Object saValue = record.getAttribute(SAMTag.SA);
         if (saValue == null) return Collections.emptyList();
         if (!(saValue instanceof String)) throw new SAMException(
                 "Expected a String for attribute 'SA' but got " + saValue.getClass() + ". Record: " + record);
@@ -1235,7 +1236,7 @@ public final class SAMUtils {
             /* fill NM */
             try {
                 if (!commaStrs[5].equals("*")) {
-                    otherRec.setAttribute(SAMTag.NM.getBinaryTag(), Integer.parseInt(commaStrs[5]));
+                    otherRec.setAttribute(SAMTag.NM, Integer.parseInt(commaStrs[5]));
                 }
             } catch (final NumberFormatException err) {
                 throw new SAMException("bad NM in " + semiColonStr + ". Record: " + record, err);
@@ -1280,17 +1281,17 @@ public final class SAMUtils {
         }
         final String oaValue;
         if (record.getReadUnmappedFlag()) {
-            oaValue = String.format("*,0,%s,*,%s,",
+            oaValue = String.format(Locale.US, "*,0,%s,*,%s,",
                     record.getReadNegativeStrandFlag() ? Strand.NEGATIVE : Strand.POSITIVE,
                     record.getMappingQuality());
         } else {
-            oaValue = String.format("%s,%s,%s,%s,%s,%s",
+            oaValue = String.format(Locale.US, "%s,%s,%s,%s,%s,%s",
                     record.getReferenceName(),
                     record.getAlignmentStart(),
                     record.getReadNegativeStrandFlag() ? Strand.NEGATIVE : Strand.POSITIVE,
                     record.getCigarString(),
                     record.getMappingQuality(),
-                    Optional.ofNullable(record.getAttribute(SAMTag.NM.name())).orElse(""));
+                    Optional.ofNullable(record.getAttribute(SAMTag.NM)).orElse(""));
         }
         return oaValue;
 
