@@ -37,6 +37,16 @@ import htsjdk.samtools.SAMRecord;
  */
 public class AbstractRecordAndOffset {
 
+    /**
+     * Classifies whether the given event is a match, insertion, or deletion. This is deliberately not expressed
+     * as CIGAR operators, since there is no knowledge of the CIGAR string at the time that this is determined.
+     */
+    public enum AlignmentType {
+        Match,
+        Insertion,
+        Deletion,
+    }
+
     /**
      * A SAMRecord aligned to reference position
      */
@@ -46,6 +56,12 @@ public class AbstractRecordAndOffset {
      */
     protected final int offset;
 
+    /**
+     * The {@link AlignmentType} of this object, which classifies whether the given event is a match, insertion, or
+     * deletion when queried from a {@link SamLocusIterator}.
+     */
+    protected final AlignmentType alignmentType;
+
     /**
      * @param record inner SAMRecord
      * @param offset from the start of the read
@@ -53,6 +69,19 @@ public class AbstractRecordAndOffset {
     public AbstractRecordAndOffset(final SAMRecord record, final int offset) {
         this.offset = offset;
         this.record = record;
+        this.alignmentType = AlignmentType.Match;
+    }
+
+    /**
+     * @param record inner SAMRecord
+     * @param offset from the start of the read
+     * @param alignmentType The {@link AlignmentType} of this object, which is used when queried in
+     *                      a {@link SamLocusIterator}.
+     */
+    public AbstractRecordAndOffset(final SAMRecord record, final int offset, final AlignmentType alignmentType) {
+        this.offset = offset;
+        this.record = record;
+        this.alignmentType = alignmentType;
     }
 
     /**
@@ -69,6 +98,14 @@ public class AbstractRecordAndOffset {
         return record;
     }
 
+    /**
+     * The {@link AlignmentType} of this object, which classifies whether the given event is a match, insertion, or
+     * deletion when queried from a {@link SamLocusIterator}.
+     */
+    public AlignmentType getAlignmentType() {
+        return alignmentType;
+    }
+
     /**
      * @return the read base according to <code>offset</code>.
      */
@@ -106,8 +143,9 @@ public class AbstractRecordAndOffset {
 
     protected void validateOffset(int offset, final byte[] array) {
         if (offset < 0 || offset >= array.length) {
-            throw new IllegalArgumentException("The requested position is not covered by this " + this.getClass().getSimpleName() +
-                    " object.");
+            throw new IllegalArgumentException("The requested position is not covered by this " + this.getClass().getSimpleName() + " object. "
+                    + "\n Offset = " + offset + " Array length = " + array.length
+                    + "\n Record is: " + getRecord().toString());
         }
     }
 }
