@@ -28,22 +28,22 @@ import htsjdk.samtools.SAMFileHeader;
 import htsjdk.samtools.SAMSequenceDictionary;
 import htsjdk.samtools.SAMSequenceRecord;
 import htsjdk.samtools.SAMTextHeaderCodec;
+import htsjdk.tribble.IntervalList.IntervalListCodec;
+import htsjdk.tribble.MutableFeature;
+import htsjdk.utils.ValidationUtils;
 
 import java.io.BufferedReader;
-import java.io.BufferedWriter;
 import java.io.File;
 import java.io.IOException;
-import java.io.Serializable;
 import java.nio.file.Path;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
-import java.util.Comparator;
 import java.util.Iterator;
 import java.util.LinkedHashSet;
 import java.util.List;
-import java.util.SortedSet;
-import java.util.TreeSet;
+import java.util.NoSuchElementException;
+import java.util.Optional;
 
 /**
  * Represents a list of intervals against a reference sequence that can be written to
@@ -65,7 +65,11 @@ import java.util.TreeSet;
  * @author Yossi Farjoun
  */
 public class IntervalList implements Iterable<Interval> {
-    public static final String INTERVAL_LIST_FILE_EXTENSION = ".interval_list";
+    /**
+     * @deprecated since June 2019 Use {@link FileExtensions#INTERVAL_LIST} instead.
+     */
+    @Deprecated
+    public static final String INTERVAL_LIST_FILE_EXTENSION = FileExtensions.INTERVAL_LIST;
 
     private final SAMFileHeader header;
     private final List<Interval> intervals = new ArrayList<>();
@@ -76,9 +80,7 @@ public class IntervalList implements Iterable<Interval> {
      * Constructs a new interval list using the supplied header information.
      */
     public IntervalList(final SAMFileHeader header) {
-        if (header == null) {
-            throw new IllegalArgumentException("SAMFileHeader must be supplied.");
-        }
+        ValidationUtils.nonNull(header, "SAMFileHeader");
         this.header = header;
     }
 
@@ -108,9 +110,9 @@ public class IntervalList implements Iterable<Interval> {
      * Adds an interval to the list of intervals.
      */
     public void add(final Interval interval) {
-        if (header.getSequence(interval.getContig()) == null) {
-            throw new IllegalArgumentException(String.format("Cannot add interval %s, contig not in header", interval.toString()));
-        }
+        ValidationUtils.nonNull(header.getSequence(interval.getContig()),
+                () -> String.format("Cannot add interval %s, contig not in header", interval.toString()));
+
         this.intervals.add(interval);
     }
 
@@ -133,7 +135,7 @@ public class IntervalList implements Iterable<Interval> {
      */
     @Deprecated
     public void sort() {
-        Collections.sort(this.intervals, new IntervalCoordinateComparator(this.header));
+        this.intervals.sort(new IntervalCoordinateComparator(this.header));
         this.header.setSortOrder(SAMFileHeader.SortOrder.coordinate);
     }
 
@@ -252,7 +254,7 @@ public class IntervalList implements Iterable<Interval> {
      * @param concatenateNames If false, the merged interval has the name of the earlier interval.  This keeps name shorter.
      */
     public static List<Interval> getUniqueIntervals(final IntervalList list, final boolean concatenateNames) {
-        return getUniqueIntervals(list, concatenateNames, false);
+        return getUniqueIntervals(list, true, concatenateNames, false);
     }
 
     //NO SIDE EFFECTS HERE!
@@ -264,6 +266,17 @@ public class IntervalList implements Iterable<Interval> {
      * @param enforceSameStrands enforce that merged intervals have the same strand, otherwise ignore.
      */
     public static List<Interval> getUniqueIntervals(final IntervalList list, final boolean concatenateNames, final boolean enforceSameStrands) {
+        return getUniqueIntervals(list, true, concatenateNames, enforceSameStrands);
+    }
+
+    /**
+     * Merges list of intervals and reduces them like htsjdk.samtools.util.IntervalList#getUniqueIntervals()
+     *
+     * @param combineAbuttingIntervals If true, intervals that are abutting will be combined into one interval.
+     * @param concatenateNames         If false, the merged interval has the name of the earlier interval.  This keeps name shorter.
+     * @param enforceSameStrands       enforce that merged intervals have the same strand, otherwise ignore.
+     */
+    public static List<Interval> getUniqueIntervals(final IntervalList list, final boolean combineAbuttingIntervals, final boolean concatenateNames, final boolean enforceSameStrands) {
 
         final List<Interval> intervals;
         if (list.getHeader().getSortOrder() != SAMFileHeader.SortOrder.coordinate) {
@@ -272,33 +285,11 @@ public class IntervalList implements Iterable<Interval> {
             intervals = list.intervals;
         }
 
+        final IntervalMergerIterator mergeringIterator = new IntervalMergerIterator(intervals.iterator(), combineAbuttingIntervals, enforceSameStrands, concatenateNames);
+
         final List<Interval> unique = new ArrayList<>();
-        final TreeSet<Interval> toBeMerged = new TreeSet<>();
-        Interval current = null;
-
-        for (final Interval next : intervals) {
-            if (current == null) {
-                toBeMerged.add(next);
-                current = next;
-            } else if (current.intersects(next) || current.abuts(next)) {
-                if (enforceSameStrands && current.isNegativeStrand() != next.isNegativeStrand()) {
-                    throw new SAMException("Strands were not equal for: " + current.toString() + " and " + next.toString());
-                }
-                toBeMerged.add(next);
-                current = new Interval(current.getContig(), current.getStart(), Math.max(current.getEnd(), next.getEnd()), current.isNegativeStrand(), null);
-            } else {
-                // Emit merged/unique interval
-                unique.add(merge(toBeMerged, concatenateNames));
-
-                // Set current == next for next iteration
-                toBeMerged.clear();
-                current = next;
-                toBeMerged.add(current);
-            }
-        }
-
-        if (!toBeMerged.isEmpty()) {
-            unique.add(merge(toBeMerged, concatenateNames));
+        while(mergeringIterator.hasNext()){
+            unique.add(mergeringIterator.next());
         }
         return unique;
     }
@@ -381,13 +372,19 @@ public class IntervalList implements Iterable<Interval> {
     }
 
     /**
-     * Merges a sorted collection of intervals and optionally concatenates unique names or takes the first name.
+     * Merges a collection of intervals and optionally concatenates unique names or takes the first name.
+     * *
+     *
+     * @param concatenateNames if true, combine the names of all the intervals with |, otherwise use the name of the first interval.
+     * @return a single interval which spans from the minimum input start position to the maximum input end position.
+     * The resulting strandedness and contig are those of the first input with no validation.
      */
-    static Interval merge(final SortedSet<Interval> intervals, final boolean concatenateNames) {
-        final String chrom = intervals.first().getContig();
-        int start = intervals.first().getStart();
-        int end = intervals.last().getEnd();
-        final boolean neg = intervals.first().isNegativeStrand();
+    static Interval merge(final Iterable<Interval> intervals, final boolean concatenateNames) {
+        final Interval first = intervals.iterator().next();
+        final String chrom = first.getContig();
+        int start = first.getStart();
+        int end = first.getEnd();
+        final boolean neg = first.isNegativeStrand();
         final LinkedHashSet<String> names = new LinkedHashSet<>();
         final String name;
 
@@ -479,14 +476,14 @@ public class IntervalList implements Iterable<Interval> {
     }
 
     /**
-     * Calls {@link #fromFile(java.io.File)} on the provided files, and returns their {@link #union(java.util.Collection)}.
+     * Calls {@link #fromFile(java.io.File)} on the provided files, and returns their {@link #concatenate(Collection)}
      */
     public static IntervalList fromFiles(final Collection<File> intervalListFiles) {
         final Collection<IntervalList> intervalLists = new ArrayList<>();
         for (final File file : intervalListFiles) {
             intervalLists.add(IntervalList.fromFile(file));
         }
-        return IntervalList.union(intervalLists);
+        return IntervalList.concatenate(intervalLists);
     }
 
     /**
@@ -498,11 +495,6 @@ public class IntervalList implements Iterable<Interval> {
      */
     public static IntervalList fromReader(final BufferedReader in) {
 
-        final int SEQUENCE_POS=0;
-        final int START_POS=1;
-        final int END_POS=2;
-        final int STRAND_POS=3;
-        final int NAME_POS=4;
         try {
             // Setup a reader and parse the header
             final StringBuilder builder = new StringBuilder(4096);
@@ -525,66 +517,17 @@ public class IntervalList implements Iterable<Interval> {
             final IntervalList list = new IntervalList(codec.decode(headerReader, "BufferedReader"));
             final SAMSequenceDictionary dict = list.getHeader().getSequenceDictionary();
 
-            //there might not be any lines after the header, in which case we should return an empty list
+            // There might not be any lines after the header, in which case we should return an empty list
             if (line == null) {
                 return list;
             }
 
+            final IntervalListCodec intervalListCodec = new IntervalListCodec(dict);
+
             // Then read in the intervals
-            final FormatUtil format = new FormatUtil();
             do {
-                if (line.trim().isEmpty()) {
-                    continue; // skip over blank lines
-                }
-
-                // Make sure we have the right number of fields
-                final String[] fields = line.split("\t");
-                if (fields.length != 5) {
-                    throw new SAMException("Invalid interval record contains " +
-                            fields.length + " fields: " + line);
-                }
-
-                // Then parse them out
-                final String seq = fields[SEQUENCE_POS];
-                final int start = format.parseInt(fields[START_POS]);
-                final int end = format.parseInt(fields[END_POS]);
-                if (start < 1) {
-                    throw new IllegalArgumentException("Coordinate less than 1: start value of " + start +
-                            " is less than 1 and thus illegal");
-                }
-
-                if (start > end + 1) {
-                    throw new IllegalArgumentException("Start value of " + start +
-                            " is greater than end + 1 for end of value: " + end +
-                            ". I'm afraid I cannot let you do that.");
-                }
-
-                final boolean negative;
-                switch (fields[STRAND_POS]) {
-                    case "-":
-                        negative = true;
-                        break;
-                    case "+":
-                        negative = false;
-                        break;
-                    default:
-                        throw new IllegalArgumentException("Invalid strand field: " + fields[STRAND_POS]);
-                }
-
-                final String name = fields[NAME_POS];
-
-                final Interval interval = new Interval(seq, start, end, negative, name);
-                final SAMSequenceRecord sequence = dict.getSequence(seq);
-                if (sequence == null) {
-                    log.warn("Ignoring interval for unknown reference: " + interval);
-                } else {
-                    final int sequenceLength = sequence.getSequenceLength();
-                    if (sequenceLength > 0 && sequenceLength < end) {
-                        throw new IllegalArgumentException("interval with end: " + end + " extends beyond end of sequence with length: " + sequenceLength);
-                    }
-
-                    list.intervals.add(interval);
-                }
+                final Optional<Interval> maybeInterval = Optional.ofNullable(intervalListCodec.decode(line));
+                maybeInterval.ifPresent(list.intervals::add);
             }
             while ((line = in.readLine()) != null);
 
@@ -594,43 +537,30 @@ public class IntervalList implements Iterable<Interval> {
         }
     }
 
+    /**
+     * Writes out the list of intervals to the supplied path.
+     *
+     * @param path a path to write to.  If exists it will be overwritten.
+     */
+    public void write(final Path path) {
+        try {
+            try (IntervalListWriter writer = new IntervalListWriter(path, this.header)) {
+                for (final Interval interval : this) {
+                    writer.write(interval);
+                }
+            }
+        } catch (final IOException ioe) {
+            throw new SAMException("Error writing out interval list to file: " + path.toAbsolutePath(), ioe);
+        }
+    }
+
     /**
      * Writes out the list of intervals to the supplied file.
      *
      * @param file a file to write to.  If exists it will be overwritten.
      */
     public void write(final File file) {
-        try (final BufferedWriter out = IOUtil.openFileForBufferedWriting(file)) {
-            final FormatUtil format = new FormatUtil();
-
-            // Write out the header
-            if (this.header != null) {
-                final SAMTextHeaderCodec codec = new SAMTextHeaderCodec();
-                codec.encode(out, this.header);
-            }
-
-            // Write out the intervals
-            for (final Interval interval : this) {
-                out.write(interval.getContig());
-                out.write('\t');
-                out.write(format.format(interval.getStart()));
-                out.write('\t');
-                out.write(format.format(interval.getEnd()));
-                out.write('\t');
-                out.write(interval.isPositiveStrand() ? '+' : '-');
-                out.write('\t');
-                if (interval.getName() != null) {
-                    out.write(interval.getName());
-                } else {
-                    out.write(".");
-                }
-                out.newLine();
-            }
-
-            out.flush();
-        } catch (final IOException ioe) {
-            throw new SAMException("Error writing out interval list to file: " + file.getAbsolutePath(), ioe);
-        }
+        this.write(file.toPath());
     }
 
     /**
@@ -643,27 +573,21 @@ public class IntervalList implements Iterable<Interval> {
 
     public static IntervalList intersection(final IntervalList list1, final IntervalList list2) {
 
-        final IntervalList result;
         // Ensure that all the sequence dictionaries agree and merge the lists
-        SequenceUtil.assertSequenceDictionariesEqual(list1.getHeader().getSequenceDictionary(),
+        SequenceUtil.assertSequenceDictionariesEqual(
+                list1.getHeader().getSequenceDictionary(),
                 list2.getHeader().getSequenceDictionary());
 
-        result = new IntervalList(list1.getHeader().clone());
+        final IntervalList result = new IntervalList(list1.getHeader().clone());
 
-        final OverlapDetector<Interval> detector = new OverlapDetector<>(0, 0);
-
-        detector.addAll(list1.getIntervals(), list1.getIntervals());
+        final OverlapDetector<Interval> detector = OverlapDetector.create(list1.getIntervals());
 
         for (final Interval i : list2.getIntervals()) {
-            final Collection<Interval> as = detector.getOverlaps(i);
-            for (final Interval j : as) {
-                final Interval tmp = i.intersect(j);
-
-                result.add(tmp);
-            }
+            detector.getOverlaps(i).stream()
+                    .map(i::intersect)
+                    .forEach(result::add);
         }
         return result.uniqued();
-
     }
 
     /**
@@ -674,44 +598,61 @@ public class IntervalList implements Iterable<Interval> {
      */
 
     public static IntervalList intersection(final Collection<IntervalList> lists) {
-
-        IntervalList intersection = null;
-        for (final IntervalList list : lists) {
-            if (intersection == null) {
-                intersection = list;
-            } else {
-                intersection = intersection(intersection, list);
-            }
-        }
-        return intersection;
+        return lists.stream()
+                .reduce(IntervalList::intersection)
+                .orElse(null);
     }
 
     /**
-     * A utility function for merging a list of IntervalLists, checks for equal dictionaries.
+     * A utility function for merging a two IntervalLists, checks for equal dictionaries.
      * Merging does not look for overlapping intervals nor uniquify
      *
+     * @param list1 the first list
+     * @param list2 the second list
+     * @return the union of all the IntervalLists in lists.
+     */
+    public static IntervalList concatenate(final IntervalList list1, final IntervalList list2) {
+
+        final SAMFileHeader header = list1.getHeader().clone();
+
+        // Ensure that all the sequence dictionaries agree and merge the lists
+        return new IntervalList(header).addOther(list1).addOther(list2);
+    }
+
+    /**
+     * A method for  concatenating the intervals from one list to this one, checks for equal dictionaries.
+     * Does not look for overlapping intervals nor uniquify.
+     *
+     * @param other the other list
+     * @return the modified this
+     */
+    public IntervalList addOther(final IntervalList other) {
+
+        SequenceUtil.assertSequenceDictionariesEqual(
+                this.getHeader().getSequenceDictionary(),
+                other.getHeader().getSequenceDictionary());
+        this.header.setSortOrder(SAMFileHeader.SortOrder.unsorted);
+        this.addall(other.intervals);
+        return this;
+    }
+
+    /**
+     * A utility function for concatenating a list of IntervalLists, checks for equal dictionaries.
+     * Concatenating does not look for overlapping intervals nor uniquify the intervals.
+     *
      * @param lists a list of IntervalList
      * @return the union of all the IntervalLists in lists.
      */
     public static IntervalList concatenate(final Collection<IntervalList> lists) {
-        if (lists.isEmpty()) {
-            throw new SAMException("Cannot concatenate an empty list of IntervalLists.");
-        }
 
-        // Ensure that all the sequence dictionaries agree and merge the lists
-        final SAMFileHeader header = lists.iterator().next().getHeader().clone();
-        header.setSortOrder(SAMFileHeader.SortOrder.unsorted);
+        final SAMFileHeader header = lists.stream()
+                .findFirst()
+                .map(IntervalList::getHeader)
+                .orElseThrow(
+                        () -> new IllegalArgumentException("Cannot combine empty collection of IntervalLists"));
 
-        final IntervalList merged = new IntervalList(header);
-
-        for (final IntervalList in : lists) {
-            SequenceUtil.assertSequenceDictionariesEqual(merged.getHeader().getSequenceDictionary(),
-                    in.getHeader().getSequenceDictionary());
-
-            merged.addall(in.intervals);
-        }
-
-        return merged;
+        return lists.stream()
+                .reduce(new IntervalList(header), IntervalList::addOther, IntervalList::concatenate);
     }
 
     /**
@@ -744,7 +685,10 @@ public class IntervalList implements Iterable<Interval> {
 
         //add all the intervals (uniqued and therefore also sorted) to a ListMap from sequenceIndex to a list of Intervals
         for (final Interval i : list.uniqued().getIntervals()) {
-            map.add(list.getHeader().getSequenceIndex(i.getContig()), i);
+            final int sequenceIndex = list.getHeader().getSequenceIndex(i.getContig());
+            ValidationUtils.validateArg(sequenceIndex != SAMSequenceRecord.UNAVAILABLE_SEQUENCE_INDEX,
+                    () -> String.format("Cannot add interval %s, contig not in header", i.toString()));
+            map.add(sequenceIndex, i);
         }
 
         // a counter to supply newly-created intervals with a name
@@ -770,8 +714,8 @@ public class IntervalList implements Iterable<Interval> {
                 }
             }
             if (sequenceLength > lastCoveredPosition) {
-            // finally, if there's space between the last interval and the next
-            // one, add an interval. This also covers the case that there are no intervals in the ListMap for a contig.
+                // finally, if there's space between the last interval and the next
+                // one, add an interval. This also covers the case that there are no intervals in the ListMap for a contig.
                 inverse.add(new Interval(sequenceName, lastCoveredPosition + 1, sequenceLength, false, "interval-" + (++intervals)));
             }
         }
@@ -779,6 +723,18 @@ public class IntervalList implements Iterable<Interval> {
         return inverse;
     }
 
+    /**
+     * A utility function for subtracting one IntervalLists from another. Resulting loci are those that are in the first
+     * but not the second.
+     *
+     * @param lhs the IntervalList from which to subtract intervals
+     * @param rhs the IntervalList to subtract
+     * @return an IntervalList comprising all loci that are in the first IntervalList but not the second  lhs-rhs=answer.
+     */
+    public static IntervalList subtract(final IntervalList lhs, final IntervalList rhs) {
+        return intersection(lhs, invert(rhs));
+    }
+
     /**
      * A utility function for subtracting a collection of IntervalLists from another. Resulting loci are those that are in the first collection
      * but not the second.
@@ -788,22 +744,9 @@ public class IntervalList implements Iterable<Interval> {
      * @return an IntervalList comprising all loci that are in the first collection but not the second  lhs-rhs=answer.
      */
     public static IntervalList subtract(final Collection<IntervalList> lhs, final Collection<IntervalList> rhs) {
-        return intersection(
+        return subtract(
                 union(lhs),
-                invert(union(rhs)));
-    }
-
-    /**
-     * A utility function for subtracting a single IntervalList from another. Resulting loci are those that are in the first List
-     * but not the second.
-     *
-     * @param lhs the IntervalList from which to subtract intervals
-     * @param rhs the IntervalList to subtract
-     * @return an IntervalList comprising all loci that are in first IntervalList but not the second. lhs-rhs=answer
-     */
-    public static IntervalList subtract(final IntervalList lhs, final IntervalList rhs) {
-        return subtract(Collections.singletonList(lhs),
-                Collections.singletonList(rhs));
+                union(rhs));
     }
 
     /**
@@ -814,9 +757,20 @@ public class IntervalList implements Iterable<Interval> {
      * @return the difference between the two intervals, i.e. the loci that are only in one IntervalList but not both
      */
     public static IntervalList difference(final Collection<IntervalList> lists1, final Collection<IntervalList> lists2) {
+        return difference(union(lists1), union(lists2));
+    }
+
+    /**
+     * A utility function for finding the difference between two IntervalLists.
+     *
+     * @param list1 the first collection of IntervalLists
+     * @param list2 the second collection of IntervalLists
+     * @return the difference between the two intervals, i.e. the loci that are only in one IntervalList but not both
+     */
+    public static IntervalList difference(final IntervalList list1, final IntervalList list2) {
         return union(
-                subtract(lists1, lists2),
-                subtract(lists2, lists1));
+                subtract(list1, list2),
+                subtract(list2, list1));
     }
 
     /**
@@ -829,7 +783,32 @@ public class IntervalList implements Iterable<Interval> {
      * any interval in the second.
      */
     public static IntervalList overlaps(final IntervalList lhs, final IntervalList rhs) {
-        return overlaps(Collections.singletonList(lhs), Collections.singletonList(rhs));
+
+        final SAMFileHeader header = lhs.getHeader().clone();
+        SequenceUtil.assertSequenceDictionariesEqual(header.getSequenceDictionary(),
+                rhs.getHeader().getSequenceDictionary());
+
+        header.setSortOrder(SAMFileHeader.SortOrder.unsorted);
+
+        // Create an overlap detector on rhs
+        final IntervalList overlapIntervals = new IntervalList(header);
+        overlapIntervals.addall(rhs.getIntervals());
+
+        final OverlapDetector<Integer> detector = new OverlapDetector<>(0, 0);
+        final int dummy = -1; // NB: since we don't actually use the returned objects, we can use a dummy value
+        for (final Interval interval : overlapIntervals.sorted().uniqued()) {
+            detector.addLhs(dummy, interval);
+        }
+
+        // Go through each input interval in lhs and see if overlaps any interval in rhs
+        final IntervalList merged = new IntervalList(header);
+        for (final Interval interval : lhs.getIntervals()) {
+            if (detector.overlapsAny(interval)) {
+                merged.add(interval);
+            }
+        }
+
+        return merged;
     }
 
     /**
@@ -845,36 +824,7 @@ public class IntervalList implements Iterable<Interval> {
         if (lists1.isEmpty()) {
             throw new SAMException("Cannot call overlaps with the first collection having empty list of IntervalLists.");
         }
-
-        final SAMFileHeader header = lists1.iterator().next().getHeader().clone();
-        header.setSortOrder(SAMFileHeader.SortOrder.unsorted);
-
-        // Create an overlap detector on list2
-        final IntervalList overlapIntervals = new IntervalList(header);
-        for (final IntervalList list : lists2) {
-            SequenceUtil.assertSequenceDictionariesEqual(header.getSequenceDictionary(),
-                    list.getHeader().getSequenceDictionary());
-            overlapIntervals.addall(list.getIntervals());
-        }
-        final OverlapDetector<Integer> detector = new OverlapDetector<>(0, 0);
-        final int dummy = -1; // NB: since we don't actually use the returned objects, we can use a dummy value
-        for (final Interval interval : overlapIntervals.sorted().uniqued()) {
-            detector.addLhs(dummy, interval);
-        }
-
-        // Go through each input interval in in lists1 and see if overlaps any interval in lists2
-        final IntervalList merged = new IntervalList(header);
-        for (final IntervalList list : lists1) {
-            SequenceUtil.assertSequenceDictionariesEqual(header.getSequenceDictionary(),
-                    list.getHeader().getSequenceDictionary());
-            for (final Interval interval : list.getIntervals()) {
-                if (detector.overlapsAny(interval)) {
-                    merged.add(interval);
-                }
-            }
-        }
-
-        return merged;
+        return overlaps(concatenate(lists1), union(lists2));
     }
 
     @Override
@@ -897,57 +847,75 @@ public class IntervalList implements Iterable<Interval> {
         result = 31 * result + intervals.hashCode();
         return result;
     }
-}
-
-/**
- * Comparator that orders intervals based on their sequence index, by coordinate
- * then by strand and finally by name.
- */
-class IntervalCoordinateComparator implements Comparator<Interval>, Serializable {
-    private static final long serialVersionUID = 1L;
-
-    private final SAMFileHeader header;
 
     /**
-     * Constructs a comparator using the supplied sequence header.
+     * An iterator that feeds on an {@link Iterator<Interval>} and combines consecutive intervals that need merging.
+     * Overlapping intervals will always be merged, abutting intervals are optionally merged.
+     *
+     * The iterator assumes that the intervals are sorted. Results are undefined if they are not sorted.
      */
-    IntervalCoordinateComparator(final SAMFileHeader header) {
-        this.header = header;
-    }
+    public static class IntervalMergerIterator implements Iterator<Interval> {
 
-    @Override
-    public int compare(final Interval lhs, final Interval rhs) {
-        final int lhsIndex = this.header.getSequenceIndex(lhs.getContig());
-        final int rhsIndex = this.header.getSequenceIndex(rhs.getContig());
-        int retval = lhsIndex - rhsIndex;
+        final Iterator<Interval> inputIntervals;
+        final boolean combineAbuttingIntervals;
+        final boolean enforceSameStrands;
+        final boolean concatenateNames;
+        final List<Interval> toBeMerged = new ArrayList<>();
 
-        if (retval == 0) {
-            retval = lhs.getStart() - rhs.getStart();
+        MutableFeature current = null;
+        boolean currentStrandNegative = false;
+
+        public IntervalMergerIterator(Iterator<Interval> intervals, final boolean combineAbuttingIntervals, final boolean enforceSameStrand, final boolean concatenateNames) {
+            this.inputIntervals = intervals;
+
+            this.combineAbuttingIntervals = combineAbuttingIntervals;
+            this.enforceSameStrands = enforceSameStrand;
+            this.concatenateNames = concatenateNames;
         }
-        if (retval == 0) {
-            retval = lhs.getEnd() - rhs.getEnd();
+
+        @Override
+        public boolean hasNext() {
+            return current != null || inputIntervals.hasNext();
         }
-        if (retval == 0) {
-            if (lhs.isPositiveStrand() && rhs.isNegativeStrand()) {
-                retval = -1;
-            } else if (lhs.isNegativeStrand() && rhs.isPositiveStrand()) {
-                retval = 1;
+
+        @Override
+        public Interval next() {
+            if (!hasNext()) {
+                throw new NoSuchElementException("There were no more elements to give!");
             }
+            return getNext();
         }
-        if (retval == 0) {
-            if (lhs.getName() == null) {
-                if (rhs.getName() == null) {
-                    return 0;
+
+        private Interval getNext() {
+            Interval next;
+            while (inputIntervals.hasNext()) {
+                next = inputIntervals.next();
+                if (current == null) {
+                    toBeMerged.add(next);
+                    current = new MutableFeature(next);
+                    currentStrandNegative = next.isNegativeStrand();
+                } else if (current.overlaps(next) || (combineAbuttingIntervals && current.withinDistanceOf(next,1))) {
+                    if (enforceSameStrands && currentStrandNegative != next.isNegativeStrand()) {
+                        throw new SAMException("Strands were not equal for: " + current.toString() + " and " + next.toString());
+                    }
+                    toBeMerged.add(next);
+                    current.end = Math.max(current.getEnd(), next.getEnd());
                 } else {
-                    return -1;
+                    // Emit merged/unique interval
+                    final Interval retVal = merge(toBeMerged, concatenateNames);
+                    toBeMerged.clear();
+                    current.setAll(next);
+                    currentStrandNegative = next.isNegativeStrand();
+                    toBeMerged.add(next);
+                    return retVal;
                 }
-            } else if (rhs.getName() == null) {
-                return 1;
-            } else {
-                return lhs.getName().compareTo(rhs.getName());
             }
+            // Emit merged/unique interval
+            final Interval retVal = merge(toBeMerged, concatenateNames);
+            toBeMerged.clear();
+            current = null;
+            return retVal;
         }
-
-        return retval;
     }
 }
+
