@@ -9,7 +9,6 @@ import htsjdk.samtools.util.Log;
 import htsjdk.tribble.AbstractFeatureCodec;
 import htsjdk.tribble.Feature;
 import htsjdk.tribble.FeatureCodecHeader;
-import htsjdk.tribble.SimpleFeature;
 import htsjdk.tribble.TribbleException;
 import htsjdk.tribble.annotation.Strand;
 import htsjdk.tribble.index.tabix.TabixFormat;
@@ -17,13 +16,13 @@ import htsjdk.tribble.readers.*;
 import htsjdk.tribble.util.ParsingUtils;
 
 
+
 import java.io.*;
 import java.net.URLDecoder;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.util.*;
 import java.util.regex.Pattern;
-import java.util.stream.Collectors;
 import java.util.zip.GZIPInputStream;
 
 /**
@@ -37,10 +36,7 @@ import java.util.zip.GZIPInputStream;
 
 public class Gff3Codec extends AbstractFeatureCodec<Gff3Feature, LineIterator> {
 
-    private static final char FIELD_DELIMITER = '\t';
-    private static final char ATTRIBUTE_DELIMITER = ';';
-    private static final char KEY_VALUE_SEPARATOR = '=';
-    private static final char VALUE_DELIMITER = ',';
+
 
     private static final int NUM_FIELDS = 9;
 
@@ -49,15 +45,12 @@ public class Gff3Codec extends AbstractFeatureCodec<Gff3Feature, LineIterator> {
     private static final int FEATURE_TYPE_INDEX = 2;
     private static final int START_LOCATION_INDEX = 3;
     private static final int END_LOCATION_INDEX = 4;
+    private static final int SCORE_INDEX = 5;
     private static final int GENOMIC_STRAND_INDEX = 6;
     private static final int GENOMIC_PHASE_INDEX = 7;
     private static final int EXTRA_FIELDS_INDEX = 8;
 
-    private static final String COMMENT_START = "#";
 
-    private static final String DIRECTIVE_START = "##";
-
-    static final String PARENT_ATTRIBUTE_KEY = "Parent";
     private static final String IS_CIRCULAR_ATTRIBUTE_KEY = "Is_circular";
 
     private static final String ARTEMIS_FASTA_MARKER = ">";
@@ -67,20 +60,38 @@ public class Gff3Codec extends AbstractFeatureCodec<Gff3Feature, LineIterator> {
     private final Map<String, Set<Gff3FeatureImpl>> activeFeaturesWithIDs = new HashMap<>();
     private final Map<String, Set<Gff3FeatureImpl>> activeParentIDs = new HashMap<>();
 
-    private int currentLineNum = 0;
-
-    private final Map<String, SequenceRegion> sequenceRegionMap = new HashMap<>();
+    private final Map<String, SequenceRegion> sequenceRegionMap = new LinkedHashMap<>();
+    private final Map<Integer, String> commentsWithLineNumbers = new LinkedHashMap<>();
 
     private final static Log logger = Log.getInstance(Gff3Codec.class);
 
     private boolean reachedFasta = false;
 
+    private DecodeDepth decodeDepth;
+
+    private int currentLine = 0;
+
     public Gff3Codec() {
+        this(DecodeDepth.DEEP);
+    }
+
+    public Gff3Codec(final DecodeDepth decodeDepth) {
         super(Gff3Feature.class);
+        this.decodeDepth = decodeDepth;
+    }
+
+    public enum DecodeDepth {
+        DEEP ,
+        SHALLOW
     }
 
     @Override
     public Gff3Feature decode(final LineIterator lineIterator) throws IOException {
+        return decode(lineIterator, decodeDepth);
+    }
+
+    private Gff3Feature decode(final LineIterator lineIterator, final DecodeDepth depth) throws IOException {
+        currentLine++;
         /*
         Basic strategy: Load features into deque, create maps from a features ID to it, and from a features parents' IDs to it.  For each feature, link to parents using these maps.
         When reaching flush directive, fasta, or end of file, prepare to flush features by moving all active features to deque of features to flush, and clearing
@@ -93,7 +104,6 @@ public class Gff3Codec extends AbstractFeatureCodec<Gff3Feature, LineIterator> {
         }
 
         final String line = lineIterator.next();
-        currentLineNum++;
 
         if (reachedFasta) {
             //previously reached fasta, flush whatever is active
@@ -107,37 +117,23 @@ public class Gff3Codec extends AbstractFeatureCodec<Gff3Feature, LineIterator> {
             return featuresToFlush.poll();
         }
 
-        if (line.startsWith(COMMENT_START) && !line.startsWith(DIRECTIVE_START)) {
+        if (line.startsWith(Gff3Constants.COMMENT_START) && !line.startsWith(Gff3Constants.DIRECTIVE_START)) {
+            commentsWithLineNumbers.put(currentLine, line.substring(Gff3Constants.COMMENT_START.length()));
             return featuresToFlush.poll();
         }
 
-        if (line.startsWith(DIRECTIVE_START)) {
+        if (line.startsWith(Gff3Constants.DIRECTIVE_START)) {
             parseDirective(line);
             return featuresToFlush.poll();
         }
 
-        final List<String> splitLine = ParsingUtils.split(line, FIELD_DELIMITER);
 
-        if (splitLine.size() != NUM_FIELDS) {
-            throw new TribbleException("Found an invalid number of columns in the given Gff3 file on line "
-                    + currentLineNum + " - Given: " + splitLine.size() + " Expected: " + NUM_FIELDS + " : " + line);
-        }
 
-        try {
-            final String contig = URLDecoder.decode(splitLine.get(CHROMOSOME_NAME_INDEX), "UTF-8");
-            final String source = URLDecoder.decode(splitLine.get(ANNOTATION_SOURCE_INDEX), "UTF-8");
-            final String type = URLDecoder.decode(splitLine.get(FEATURE_TYPE_INDEX), "UTF-8");
-            final int start = Integer.parseInt(splitLine.get(START_LOCATION_INDEX));
-            final int end = Integer.parseInt(splitLine.get(END_LOCATION_INDEX));
-            final int phase = splitLine.get(GENOMIC_PHASE_INDEX).equals(".")? -1 : Integer.parseInt(splitLine.get(GENOMIC_PHASE_INDEX));
-            final Strand strand = Strand.decode(splitLine.get(GENOMIC_STRAND_INDEX));
-            final Map<String, String> attributes = parseAttributes(splitLine.get(EXTRA_FIELDS_INDEX));
-
-            final String parentIDAttribute = attributes.get(PARENT_ATTRIBUTE_KEY);
-            final List<String> parentIDs = parentIDAttribute != null? ParsingUtils.split(parentIDAttribute, VALUE_DELIMITER) : new ArrayList<>();
-
-            final Gff3FeatureImpl thisFeature = new Gff3FeatureImpl(contig, source, type, start, end, strand, phase, attributes);
-            activeFeatures.add(thisFeature);
+        final Gff3FeatureImpl thisFeature = new Gff3FeatureImpl(parseLine(line, currentLine));
+        activeFeatures.add(thisFeature);
+        if (depth == DecodeDepth.DEEP) {
+            //link to parents/children/co-features
+            final List<String> parentIDs = thisFeature.getAttribute(Gff3Constants.PARENT_ATTRIBUTE_KEY);
             final String id = thisFeature.getID();
 
             for (final String parentID : parentIDs) {
@@ -171,11 +167,14 @@ public class Gff3Codec extends AbstractFeatureCodec<Gff3Feature, LineIterator> {
                     child.addParent(thisFeature);
                 }
             }
-            validateFeature(thisFeature);
-            return featuresToFlush.poll();
-        } catch( final NumberFormatException ex ){
-            throw new TribbleException("Cannot read integer value for start/end position!", ex);
         }
+
+        validateFeature(thisFeature);
+        if (depth == DecodeDepth.SHALLOW) {
+            //flush all features immediatly
+            prepareToFlushFeatures();
+        }
+        return featuresToFlush.poll();
     }
 
 
@@ -185,19 +184,71 @@ public class Gff3Codec extends AbstractFeatureCodec<Gff3Feature, LineIterator> {
      * @return map of keys to values for attributes of this feature
      * @throws UnsupportedEncodingException
      */
-    static private Map<String,String> parseAttributes(final String attributesString) throws UnsupportedEncodingException {
-        final Map<String, String> attributes = new LinkedHashMap<>();
-        final List<String> splitLine = ParsingUtils.split(attributesString,ATTRIBUTE_DELIMITER);
+    static private Map<String, List<String>> parseAttributes(final String attributesString) throws UnsupportedEncodingException {
+        if (attributesString.equals(Gff3Constants.UNDEFINED_FIELD_VALUE)) {
+            return Collections.emptyMap();
+        }
+        final Map<String, List<String>> attributes = new LinkedHashMap<>();
+        final List<String> splitLine = ParsingUtils.split(attributesString,Gff3Constants.ATTRIBUTE_DELIMITER);
         for(String attribute : splitLine) {
-            final List<String> key_value = ParsingUtils.split(attribute,KEY_VALUE_SEPARATOR);
-            if (key_value.size()<2) {
-                continue;
+            final List<String> key_value = ParsingUtils.split(attribute,Gff3Constants.KEY_VALUE_SEPARATOR);
+            if (key_value.size() != 2) {
+                throw new TribbleException("Attribute string " + attributesString + " is invalid");
             }
-            attributes.put(URLDecoder.decode(key_value.get(0).trim(), "UTF-8"), URLDecoder.decode(key_value.get(1).trim(), "UTF-8"));
+            attributes.put(URLDecoder.decode(key_value.get(0).trim(), "UTF-8"), decodeAttributeValue(key_value.get(1).trim()));
         }
         return attributes;
     }
 
+    static private Gff3BaseData parseLine(final String line, final int currentLine) {
+        final List<String> splitLine = ParsingUtils.split(line, Gff3Constants.FIELD_DELIMITER);
+
+        if (splitLine.size() != NUM_FIELDS) {
+            throw new TribbleException("Found an invalid number of columns in the given Gff3 file at line + " + currentLine + " - Given: " + splitLine.size() + " Expected: " + NUM_FIELDS + " : " + line);
+        }
+
+        try {
+            final String contig = URLDecoder.decode(splitLine.get(CHROMOSOME_NAME_INDEX), "UTF-8");
+            final String source = URLDecoder.decode(splitLine.get(ANNOTATION_SOURCE_INDEX), "UTF-8");
+            final String type = URLDecoder.decode(splitLine.get(FEATURE_TYPE_INDEX), "UTF-8");
+            final int start = Integer.parseInt(splitLine.get(START_LOCATION_INDEX));
+            final int end = Integer.parseInt(splitLine.get(END_LOCATION_INDEX));
+            final double score = splitLine.get(SCORE_INDEX).equals(Gff3Constants.UNDEFINED_FIELD_VALUE) ? -1 : Double.parseDouble(splitLine.get(SCORE_INDEX));
+            final int phase = splitLine.get(GENOMIC_PHASE_INDEX).equals(Gff3Constants.UNDEFINED_FIELD_VALUE) ? -1 : Integer.parseInt(splitLine.get(GENOMIC_PHASE_INDEX));
+            final Strand strand = Strand.decode(splitLine.get(GENOMIC_STRAND_INDEX));
+            final Map<String, List<String>> attributes = parseAttributes(splitLine.get(EXTRA_FIELDS_INDEX));
+            return new Gff3BaseData(contig, source, type, start, end, score, strand, phase, attributes);
+        } catch (final NumberFormatException ex ) {
+            throw new TribbleException("Cannot read integer value for start/end position from line " + currentLine + ".  Line is: " + line, ex);
+        } catch (final IOException ex) {
+            throw new TribbleException("Cannot decode feature info from line " + currentLine + ".  Line is: " + line, ex);
+        }
+    }
+
+    /**
+     * Get list of sequence regions parsed by the codec.
+     * @return list of sequence regions
+     */
+    public List<SequenceRegion> getSequenceRegions() {
+        return Collections.unmodifiableList(new ArrayList<>(sequenceRegionMap.values()));
+    }
+
+    /**
+     * Gets map from line number to comment found on that line.  The text of the comment EXCLUDES the leading # which indicates a comment line.
+     * @return Map from line number to comment found on line
+     */
+    public Map<Integer, String> getCommentsWithLineNumbers() {
+        return Collections.unmodifiableMap(new LinkedHashMap<>(commentsWithLineNumbers));
+    }
+
+    /**
+     * Gets list of comments parsed by the codec.  Excludes leading # which indicates a comment line.
+     * @return
+     */
+    public List<String> getCommentTexts() {
+        return Collections.unmodifiableList(new ArrayList<>(commentsWithLineNumbers.values()));
+    }
+
     /**
      * If sequence region of feature's contig has been specified with sequence region directive, validates that
      * feature's coordinates are within the specified sequence region.  TribbleException is thrown if invalid.
@@ -208,7 +259,7 @@ public class Gff3Codec extends AbstractFeatureCodec<Gff3Feature, LineIterator> {
             final SequenceRegion region = sequenceRegionMap.get(feature.getContig());
             if (feature.getStart() == region.getStart() && feature.getEnd() == region.getEnd()) {
                 //landmark feature
-                final boolean isCircular = Boolean.parseBoolean(feature.getAttribute(IS_CIRCULAR_ATTRIBUTE_KEY));
+                final boolean isCircular = Boolean.parseBoolean(extractSingleAttribute(feature.getAttribute(IS_CIRCULAR_ATTRIBUTE_KEY)));
                 region.setCircular(isCircular);
             }
             if (region.isCircular()? !region.overlaps(feature) : !region.contains(feature)) {
@@ -219,20 +270,8 @@ public class Gff3Codec extends AbstractFeatureCodec<Gff3Feature, LineIterator> {
     }
 
     @Override
-    public Feature decodeLoc(LineIterator lineIterator) {
-        final String line = lineIterator.next();
-
-        if (line.startsWith(COMMENT_START)) {
-            return null;
-        }
-
-        final List<String> splitLine = ParsingUtils.split(line,FIELD_DELIMITER);
-
-        try {
-            return new SimpleFeature(splitLine.get(CHROMOSOME_NAME_INDEX), Integer.parseInt(splitLine.get(START_LOCATION_INDEX)), Integer.parseInt(splitLine.get(END_LOCATION_INDEX)));
-        } catch (final NumberFormatException ex ) {
-            throw new TribbleException("Cannot read integer value for start/end position!", ex);
-        }
+    public Feature decodeLoc(LineIterator lineIterator) throws IOException {
+        return decode(lineIterator, DecodeDepth.SHALLOW);
     }
 
     @Override
@@ -256,7 +295,7 @@ public class Gff3Codec extends AbstractFeatureCodec<Gff3Feature, LineIterator> {
                     if (Gff3Directive.toDirective(line) != Gff3Directive.VERSION3_DIRECTIVE) {
                         return false;
                     }
-                    while (line.startsWith(COMMENT_START)) {
+                    while (line.startsWith(Gff3Constants.COMMENT_START)) {
                         line = br.readLine();
                         if ( line == null ) {
                             return false;
@@ -264,7 +303,7 @@ public class Gff3Codec extends AbstractFeatureCodec<Gff3Feature, LineIterator> {
                     }
 
                     // make sure line conforms to gtf spec
-                    final List<String> fields = ParsingUtils.split(line,FIELD_DELIMITER);
+                    final List<String> fields = ParsingUtils.split(line, Gff3Constants.FIELD_DELIMITER);
 
                     canDecode &= fields.size() == NUM_FIELDS;
 
@@ -300,13 +339,40 @@ public class Gff3Codec extends AbstractFeatureCodec<Gff3Feature, LineIterator> {
         return canDecode;
     }
 
+    static List<String> decodeAttributeValue(final String attributeValue) {
+        //split on VALUE_DELIMITER, then decode
+        final List<String> splitValues = ParsingUtils.split(attributeValue, Gff3Constants.VALUE_DELIMITER);
+
+        final List<String> decodedValues = new ArrayList<>();
+        for (final String encodedValue : splitValues) {
+            try {
+                decodedValues.add(URLDecoder.decode(encodedValue.trim(), "UTF-8"));
+            } catch (final UnsupportedEncodingException ex) {
+                throw new TribbleException("Error decoding attribute " + encodedValue, ex);
+            }
+        }
+
+        return decodedValues;
+    }
+
+    static String extractSingleAttribute(final List<String> values) {
+        if (values == null || values.isEmpty()) {
+            return null;
+        }
+
+        if (values.size() != 1) {
+            throw new TribbleException("Attribute has multiple values when only one expected");
+        }
+        return values.get(0);
+    }
+
     @Override
     public FeatureCodecHeader readHeader(LineIterator lineIterator) {
 
         List<String> header = new ArrayList<>();
         while(lineIterator.hasNext()) {
             String line = lineIterator.peek();
-            if (line.startsWith(COMMENT_START)) {
+            if (line.startsWith(Gff3Constants.COMMENT_START)) {
                 header.add(line);
                 lineIterator.next();
             } else {
@@ -407,29 +473,75 @@ public class Gff3Codec extends AbstractFeatureCodec<Gff3Feature, LineIterator> {
     /**
      * Enum for parsing directive lines.  If information in directive line needs to be parsed beyond specifying directive type, decode method should be overriden
      */
-    enum Gff3Directive {
+    public enum Gff3Directive {
 
-        VERSION3_DIRECTIVE("##gff-version\\s+3(?:.\\d)*(?:\\.\\d)*$"),
+        VERSION3_DIRECTIVE("##gff-version\\s+3(?:\\.\\d*)*$") {
+            @Override
+             protected Object decode(final String line) throws IOException {
+                final String[] splitLine = line.split("\\s+");
+                return splitLine[1];
+            }
+
+            @Override
+            String encode(final Object object) {
+                if (object == null) {
+                    throw new TribbleException("Cannot encode null in VERSION3_DIRECTIVE");
+                }
+                if (!(object instanceof String)) {
+                    throw new TribbleException("Cannot encode object of type " + object.getClass() + " in VERSION3_DIRECTIVE");
+                }
+
+                final String versionLine = "##gff-version " + (String)object;
+                if (!regexPattern.matcher(versionLine).matches()) {
+                    throw new TribbleException("Version " + (String)object + " is not a valid version");
+                }
+
+                return versionLine;
+            }
+        },
 
         SEQUENCE_REGION_DIRECTIVE("##sequence-region\\s+.+ \\d+ \\d+$") {
-            private int CONTIG_INDEX = 1;
-            private int START_INDEX = 2;
-            private int END_INDEX = 3;
+            final private int CONTIG_INDEX = 1;
+            final private int START_INDEX = 2;
+            final private int END_INDEX = 3;
             @Override
-            public Object decode(final String line) throws IOException {
+            protected Object decode(final String line) throws IOException {
                 final String[] splitLine = line.split("\\s+");
                 final String contig = URLDecoder.decode(splitLine[CONTIG_INDEX], "UTF-8");
                 final int start = Integer.parseInt(splitLine[START_INDEX]);
                 final int end = Integer.parseInt(splitLine[END_INDEX]);
                 return new SequenceRegion(contig, start, end);
             }
+
+            @Override
+            String encode(final Object object) {
+                if (object == null) {
+                    throw new TribbleException("Cannot encode null in SEQUENCE_REGION_DIRECTIVE");
+                }
+                if (!(object instanceof SequenceRegion)) {
+                    throw new TribbleException("Cannot encode object of type " + object.getClass() + " in SEQUENCE_REGION_DIRECTIVE");
+                }
+
+                final SequenceRegion sequenceRegion = (SequenceRegion) object;
+                return "##sequence-region " + Gff3Writer.encodeString(sequenceRegion.getContig()) + " " + sequenceRegion.getStart() + " " + sequenceRegion.getEnd();
+            }
         },
 
-        FLUSH_DIRECTIVE("###$"),
+        FLUSH_DIRECTIVE("###$") {
+            @Override
+            String encode(final Object object) {
+                return "###";
+            }
+        },
 
-        FASTA_DIRECTIVE("##FASTA$");
+        FASTA_DIRECTIVE("##FASTA$") {
+            @Override
+            String encode(final Object object) {
+                return "##FASTA";
+            }
+        };
 
-        private final Pattern regexPattern;
+        protected final Pattern regexPattern;
 
         Gff3Directive(String regex) {
             this.regexPattern = Pattern.compile(regex);
@@ -444,9 +556,11 @@ public class Gff3Codec extends AbstractFeatureCodec<Gff3Feature, LineIterator> {
             return null;
         }
 
-        public Object decode(final String line) throws IOException {
+        protected Object decode(final String line) throws IOException {
             return null;
         }
+
+        abstract String encode(final Object object);
     }
 
 }
