@@ -28,8 +28,10 @@ package htsjdk.variant.variantcontext.writer;
 import htsjdk.samtools.Defaults;
 import htsjdk.samtools.SAMSequenceDictionary;
 import htsjdk.samtools.util.BlockCompressedOutputStream;
+import htsjdk.samtools.util.FileExtensions;
 import htsjdk.samtools.util.IOUtil;
 import htsjdk.samtools.util.Md5CalculatingOutputStream;
+import htsjdk.samtools.util.Log;
 import htsjdk.samtools.util.RuntimeIOException;
 import htsjdk.tribble.index.IndexCreator;
 import htsjdk.tribble.index.tabix.TabixFormat;
@@ -107,6 +109,7 @@ public class VariantContextWriterBuilder {
     public static final EnumSet<Options> DEFAULT_OPTIONS = EnumSet.of(Options.INDEX_ON_THE_FLY);
     public static final EnumSet<Options> NO_OPTIONS = EnumSet.noneOf(Options.class);
     private static final OpenOption[] EMPTY_OPEN_OPTION_ARRAY = new OpenOption[0];
+    private static final Log log = Log.getInstance(VariantContextWriter.class);
 
     public enum OutputType {
         UNSPECIFIED,
@@ -453,7 +456,12 @@ public class VariantContextWriterBuilder {
 
         switch (typeToBuild) {
             case UNSPECIFIED:
-                throw new IllegalArgumentException("Must specify file or stream output type.");
+                throw new IllegalArgumentException(
+                     "Output format type is not set, or could not be inferred from the output path. "
+                     + "If a path was used, does it have a valid VCF extension ("
+                     + String.join(", ", FileExtensions.VCF_LIST)
+                     + ")?"
+                );
             case VCF:
                 if ((refDict == null) && (options.contains(Options.INDEX_ON_THE_FLY)))
                     throw new IllegalArgumentException("A reference dictionary is required for creating Tribble indices on the fly");
@@ -476,14 +484,13 @@ public class VariantContextWriterBuilder {
                 writer = createBCFWriter(outPath, outStreamFromFile);
                 break;
             case VCF_STREAM:
-                if (options.contains(Options.INDEX_ON_THE_FLY))
-                    throw new IllegalArgumentException("VCF index creation not supported for stream output.");
-
                 writer = createVCFWriter(null, outStreamFromFile);
                 break;
             case BCF_STREAM:
-                if (options.contains(Options.INDEX_ON_THE_FLY))
-                    throw new IllegalArgumentException("BCF index creation not supported for stream output.");
+                if (options.contains(Options.INDEX_ON_THE_FLY)) {
+                    log.warn("BCF index creation not supported for stream output, index will not be created");
+                    options.remove(Options.INDEX_ON_THE_FLY);
+                }
 
                 writer = createBCFWriter(null, outStream);
                 break;
@@ -547,11 +554,11 @@ public class VariantContextWriterBuilder {
     }
 
     private static boolean isVCF(final Path outPath) {
-        return outPath != null && outPath.getFileName().toString().endsWith(IOUtil.VCF_FILE_EXTENSION);
+        return outPath != null && outPath.getFileName().toString().endsWith(FileExtensions.VCF);
     }
 
     private static boolean isBCF(final Path outPath) {
-        return outPath != null && outPath.getFileName().toString().endsWith(IOUtil.BCF_FILE_EXTENSION);
+        return outPath != null && outPath.getFileName().toString().endsWith(FileExtensions.BCF);
     }
 
     private static boolean isCompressedVCF(final Path outPath) {
