@@ -3,22 +3,19 @@ package htsjdk.samtools;
 import htsjdk.HtsjdkTest;
 import com.google.common.jimfs.Configuration;
 import com.google.common.jimfs.Jimfs;
-import htsjdk.samtools.cram.build.CramIO;
 import htsjdk.samtools.cram.common.CramVersions;
 import htsjdk.samtools.cram.ref.ReferenceSource;
 import htsjdk.samtools.seekablestream.SeekableStream;
-import htsjdk.samtools.util.Log;
+import htsjdk.samtools.util.FileExtensions;
 
 import htsjdk.samtools.util.SequenceUtil;
 import java.nio.file.*;
 import org.testng.Assert;
-import org.testng.annotations.BeforeTest;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
 import java.io.*;
 
-import java.security.NoSuchAlgorithmException;
 import java.util.ArrayList;
 import java.util.List;
 
@@ -53,7 +50,7 @@ public class CRAMComplianceTest extends HtsjdkTest {
                 {"xx#tlen2"},       // SAMRecord mismatch: https://github.com/samtools/htsjdk/issues/1189
                 {"xx#triplet"},     // the version 2.1 variant of this file has a bad insertSize, which is
                                     // probably residual detritus from https://github.com/samtools/htsjdk/issues/364
-                //{"md#1"},           // fails with "offensive record" errors: https://github.com/samtools/htsjdk/issues/1187
+                {"md#1"},           // fails with "offensive record" errors: https://github.com/samtools/htsjdk/issues/1187
         };
     }
 
@@ -67,6 +64,8 @@ public class CRAMComplianceTest extends HtsjdkTest {
     @DataProvider(name = "fullVerification")
     public Object[][] getFullVerificationData() {
         return new Object[][] {
+                // TODO: this file has reads that are mapped beyond the bounds of the reference length that
+                // is specified in the embedded sequence dictionary.
                 {"c1#bounds"},
                 {"c1#clip"},
                 {"c1#pad1"},
@@ -91,42 +90,7 @@ public class CRAMComplianceTest extends HtsjdkTest {
 
     @Test(dataProvider = "fullVerification")
     public void fullVerificationTest(String name) throws IOException {
-        doComplianceTest(name, (version, expected, actual) -> Assert.assertEquals(expected, actual));
-    }
-
-    // Files that can be subjected to full verification only after read base normalization, because either
-    // the reference or the reads contain ambiguity codes that are normalized by SequenceUtil.toBamReadBasesInPlace
-    // during the round-trip process.
-    @DataProvider(name = "ambiguityCodeVerification")
-    public Object[][] getAmbiguityCodeVerificationData() {
-        return new Object[][]{
-                {"amb#amb"}
-        };
-    }
-
-    @Test(dataProvider = "ambiguityCodeVerification")
-    public void ambiguityCodeVerificationTest(String name) throws IOException {
-        doComplianceTest(name,
-                (version, expected, actual) ->
-                {
-                    if (expected.getReadString().equals(actual.getReadString())) {
-                        Assert.assertEquals(expected, actual);
-                    } else {
-                        // tolerate BAM and CRAM conversion of read bases to upper case IUPAC codes by
-                        // creating a deep copy of the expected reads and normalizing (upper case IUPAC)
-                        // the bases; then proceeding with the full compare with the actual
-                        SAMRecord expectedNormalized = actual.deepCopy();
-                        final byte[] expectedBases = expectedNormalized.getReadBases();
-                        SequenceUtil.toBamReadBasesInPlace(expectedBases);
-                        Assert.assertEquals(actual, expectedNormalized);
-                    }
-                }
-                );
-    }
-
-    @BeforeTest
-    public void beforeTest() {
-        Log.setGlobalLogLevel(Log.LogLevel.ERROR);
+        doComplianceTest(name, (version, actual, expected) -> Assert.assertEquals(actual, expected));
     }
 
     private static class TestCase {
@@ -143,19 +107,71 @@ public class CRAMComplianceTest extends HtsjdkTest {
         }
     }
 
+    // Files that can be subjected to full verification only after read base normalization, because either
+    // the reference or the reads contain ambiguity codes that are normalized by SequenceUtil.toBamReadBasesInPlace
+    // during the round-trip process.
+    @DataProvider(name = "ambiguityCodeVerification")
+    public Object[][] getAmbiguityCodeVerificationData() {
+        return new Object[][]{
+                // This test case has no 2.1 cram file because the test case had one read that had the unmapped
+                // flag set, along with a (non-zero and non-255) mapping quality value. Since a mapping quality
+                // on an unmapped read does not not roundtrip through CRAM (see
+                // https://github.com/samtools/htsjdk/issues/714), and we don't have an easy way to create a
+                // repaired 2.1 cram, the 2.1 file has just been removed from the repo.
+                {"amb#amb"}
+        };
+    }
+
+    @Test(dataProvider = "ambiguityCodeVerification")
+    public void ambiguityCodeVerificationTest(String name) throws IOException {
+        TestCase t = new TestCase(new File("src/test/resources/htsjdk/samtools/cram/"), name);
+        /// round trip the bam through cram, but tolerate ambiguity code transforms
+        roundTripTolerateAmbiguityCodeConversion(t.bamFile, t.refFile);
+        // round trip the original cram (which has already had ambiguity code transform) through bam
+        testCRAMThroughBAMRoundTrip(t.cramFile_30, t.refFile);
+    }
+
+    private void roundTripTolerateAmbiguityCodeConversion(final File inputFile, final File referenceFile) throws IOException {
+        final List<SAMRecord> originalSAMRecords = getSAMRecordsFromFile(inputFile, referenceFile);
+        // roundtrip the SAM records through a temporary CRAM
+        final File tempCRAMFile = File.createTempFile("testAmbiguousBasesBAMThroughCRAMRoundTrip", FileExtensions.CRAM);
+        tempCRAMFile.deleteOnExit();
+        final SAMFileHeader samHeader = getFileHeader(inputFile, referenceFile);
+        writeRecordsToFile(originalSAMRecords, tempCRAMFile, referenceFile, samHeader);
+        final List<SAMRecord> roundTripCRAMRecords = getSAMRecordsFromFile(tempCRAMFile, referenceFile);
+        Assert.assertEquals(roundTripCRAMRecords.size(), originalSAMRecords.size());
+
+        for (int i = 0; i < originalSAMRecords.size(); i++) {
+            final SAMRecord originalRecord = originalSAMRecords.get(i);
+            final SAMRecord roundTripRecord = roundTripCRAMRecords.get(i);
+            if (originalRecord.getReadString().equals(roundTripRecord.getReadString())) {
+                Assert.assertEquals(roundTripRecord, originalRecord);
+            } else {
+                // tolerate BAM and CRAM conversion of read bases to upper case IUPAC codes by
+                // creating a deep copy of the expected reads and normalizing (upper case IUPAC)
+                // the bases; then proceeding with the full compare with the actual
+                final SAMRecord expectedNormalized = originalRecord.deepCopy();
+                final byte[] expectedBases = expectedNormalized.getReadBases();
+                SequenceUtil.toBamReadBasesInPlace(expectedBases);
+                Assert.assertEquals(roundTripRecord, expectedNormalized);
+            }
+        }
+    }
+
     private void doComplianceTest(
             final String name,
             final TriConsumer<Integer, SAMRecord, SAMRecord> assertFunction) throws IOException {
-        TestCase t = new TestCase(new File("src/test/resources/htsjdk/samtools/cram/"), name);
+        final TestCase t = new TestCase(new File("src/test/resources/htsjdk/samtools/cram/"), name);
 
+        // 1) Read from SAM/BAM Round Trip through CRAM
         // retrieve all records from the original file
-        List<SAMRecord> samRecords = getSAMRecordsFromFile(t.bamFile, t.refFile);
-        SAMFileHeader samFileHeader = getFileHeader(t.bamFile, t.refFile);
+        final List<SAMRecord> samRecords = getSAMRecordsFromFile(t.bamFile, t.refFile);
+        final SAMFileHeader samFileHeader = getFileHeader(t.bamFile, t.refFile);
 
-        // write them to cram stream
-        ByteArrayOutputStream baos = new ByteArrayOutputStream();
-        ReferenceSource source = new ReferenceSource(t.refFile);
-        CRAMFileWriter cramFileWriter = new CRAMFileWriter(baos, source, samFileHeader, name);
+        // write them to a cram stream
+        final ByteArrayOutputStream baos = new ByteArrayOutputStream();
+        final ReferenceSource source = new ReferenceSource(t.refFile);
+        final CRAMFileWriter cramFileWriter = new CRAMFileWriter(baos, source, samFileHeader, name);
         for (SAMRecord samRecord : samRecords) {
             cramFileWriter.addAlignment(samRecord);
         }
@@ -168,53 +184,53 @@ public class CRAMComplianceTest extends HtsjdkTest {
             Assert.assertTrue(cramFileReaderIterator.hasNext());
             SAMRecord restored = cramFileReaderIterator.next();
             Assert.assertNotNull(restored);
-            assertFunction.accept(CramVersions.DEFAULT_CRAM_VERSION.major, samRecord, restored);
+            assertFunction.accept(CramVersions.DEFAULT_CRAM_VERSION.getMajor(), restored, samRecord);
         }
         Assert.assertFalse(cramFileReaderIterator.hasNext());
 
-        //v2.1 test
+        // Read from v2.1 CRAM round trip through cram
         cramFileReader = new CRAMFileReader(new FileInputStream(t.cramFile_21), (SeekableStream) null, source, ValidationStringency.SILENT);
         cramFileReaderIterator = cramFileReader.getIterator();
         for (SAMRecord samRecord : samRecords) {
             Assert.assertTrue(cramFileReaderIterator.hasNext());
             SAMRecord restored = cramFileReaderIterator.next();
             Assert.assertNotNull(restored);
-            assertFunction.accept(CramVersions.CRAM_v2_1.major, samRecord, restored);
+            assertFunction.accept(CramVersions.CRAM_v2_1.getMajor(), restored, samRecord);
         }
         Assert.assertFalse(cramFileReaderIterator.hasNext());
 
-        //v3.0 test
+        // Read from v3.0 CRAM round trip through cram
         cramFileReader = new CRAMFileReader(new FileInputStream(t.cramFile_30), (SeekableStream) null, source, ValidationStringency.SILENT);
         cramFileReaderIterator = cramFileReader.getIterator();
         for (SAMRecord samRecord : samRecords) {
             Assert.assertTrue(cramFileReaderIterator.hasNext());
             SAMRecord restored = cramFileReaderIterator.next();
             Assert.assertNotNull(restored);
-            assertFunction.accept(CramVersions.CRAM_v3.major, samRecord, restored);
+            assertFunction.accept(CramVersions.CRAM_v3.getMajor(), restored, samRecord);
         }
         Assert.assertFalse(cramFileReaderIterator.hasNext());
     }
 
-    private void assertSameRecordsPartial(Integer majorVersion, SAMRecord record1, SAMRecord record2) {
+    private void assertSameRecordsPartial(Integer majorVersion, SAMRecord actual, SAMRecord expected) {
         // test a partial set of fields for equality, avoiding known CRAM conversion issues
-        Assert.assertEquals(record2.getFlags(), record1.getFlags());
-        Assert.assertEquals(record2.getReadName(), record1.getReadName());
-        Assert.assertEquals(record2.getReferenceName(), record1.getReferenceName());
-        Assert.assertEquals(record2.getAlignmentStart(), record1.getAlignmentStart());
+        Assert.assertEquals(actual.getFlags(), expected.getFlags());
+        Assert.assertEquals(actual.getReadName(), expected.getReadName());
+        Assert.assertEquals(actual.getReferenceName(), expected.getReferenceName());
+        Assert.assertEquals(actual.getAlignmentStart(), expected.getAlignmentStart());
 
         /**
          * Known issue: CRAM v2.1 doesn't handle reads with missing bases correctly. This
          * causes '*' bases to arise when reading CRAM. Skipping the base comparison asserts.
          * https://github.com/samtools/htsjdk/issues/509
          */
-        if (record1.getReadBases() != SAMRecord.NULL_SEQUENCE || majorVersion >= CramVersions.CRAM_v3.major) {
+        if (expected.getReadBases() != SAMRecord.NULL_SEQUENCE || majorVersion >= CramVersions.CRAM_v3.getMajor()) {
             // BAM and CRAM convert read bases to upper case IUPAC codes
-            final byte[] originalBases = record1.getReadBases();
-            SequenceUtil.toBamReadBasesInPlace(originalBases);
-            Assert.assertEquals(record2.getReadBases(), originalBases);
+            final byte[] expectedBases = expected.getReadBases();
+            SequenceUtil.toBamReadBasesInPlace(expectedBases);
+            Assert.assertEquals(actual.getReadBases(), expectedBases);
         }
 
-        Assert.assertEquals(record2.getBaseQualities(), record1.getBaseQualities());
+        Assert.assertEquals(actual.getBaseQualities(), expected.getBaseQualities());
     }
 
     @DataProvider(name = "CRAMSourceFiles")
@@ -249,7 +265,7 @@ public class CRAMComplianceTest extends HtsjdkTest {
         SAMFileHeader samHeader;
         List<SAMRecord> bamRecords;
         try (FileSystem jimfs = Jimfs.newFileSystem(Configuration.unix())) {
-            final Path tempBam = jimfs.getPath("testCRAMToBAMToCRAM" + BamFileIoUtils.BAM_FILE_EXTENSION);
+            final Path tempBam = jimfs.getPath("testCRAMToBAMToCRAM" + FileExtensions.BAM);
             samHeader = getFileHeader(originalCRAMFile, referenceFile);
             writeRecordsToPath(copiedCRAMRecords, tempBam, referenceFile, samHeader);
             bamRecords = getSAMRecordsFromPath(tempBam, referenceFile);
@@ -258,7 +274,6 @@ public class CRAMComplianceTest extends HtsjdkTest {
         // compare to originals
         int i = 0;
         for (SAMRecord rec : bamRecords) {
-            rec.setIndexingBin(null);
             Assert.assertTrue(rec.equals(originalCRAMRecords.get(i++)));
         }
         Assert.assertEquals(i, originalCRAMRecords.size());
@@ -266,7 +281,7 @@ public class CRAMComplianceTest extends HtsjdkTest {
         // write the BAM records to a CRAM and read them back in
         List<SAMRecord> roundTripCRAMRecords;
         try (FileSystem jimfs = Jimfs.newFileSystem(Configuration.unix())) {
-            final Path tempCRAM = jimfs.getPath("testCRAMToBAMToCRAM" + CramIO.CRAM_FILE_EXTENSION);
+            final Path tempCRAM = jimfs.getPath("testCRAMToBAMToCRAM" + FileExtensions.CRAM);
             writeRecordsToPath(bamRecords, tempCRAM, referenceFile, samHeader);
             roundTripCRAMRecords = getSAMRecordsFromPath(tempCRAM, referenceFile);
         }
@@ -279,27 +294,33 @@ public class CRAMComplianceTest extends HtsjdkTest {
         Assert.assertEquals(i, originalCRAMRecords.size());
     }
 
-    @Test
-    public void testBAMThroughCRAMRoundTrip() throws IOException, NoSuchAlgorithmException {
+    @DataProvider(name="roundTripTest")
+    public Object[][] roundTripTestData() {
+        return new Object[][] {
+                // This file is a reduced version of the CEUTrio.HiSeq.WGS.b37.NA12878.20.21.bam and human_g1k_v37.20.21.fasta
+                // files used in GATK4 tests. The first 8000 records from chr20 were extracted; from those around 80 placed but
+                // unmapped reads that contained cigar elements were removed, along with one read who's mate was on chr21.
+                // Finally all read positions were remapped to the subsetted reference file, which contains only the ~9000 bases
+                // used by the reduced read set.
+                { "CEUTrio.HiSeq.WGS.b37.NA12878.20.first.8000.bam", "human_g1k_v37.20.subset.fasta" },
+
+                // Test that we round-trip read group ids, since these are treated specially and not round-tripped as
+                // through the general tag facility, but instead have a dedicated data series and read group.
+                { "c1WithRG.bam", "c1.fa"}
+        };
+    }
+
+    @Test(dataProvider = "roundTripTest")
+    public void testBAMThroughCRAMRoundTrip(final String testFileName, final String referenceFileName) throws IOException {
         final File TEST_DATA_DIR = new File("src/test/resources/htsjdk/samtools/cram");
 
-        // These files are reduced versions of the CEUTrio.HiSeq.WGS.b37.NA12878.20.21.bam and human_g1k_v37.20.21.fasta
-        // files used in GATK4 tests. The first 8000 records from chr20 were extracted; from those around 80 placed but
-        // unmapped reads that contained cigar elements were removed, along with one read who's mate was on chr21.
-        // Finally all read positions were remapped to the subsetted reference file, which contains only the ~9000 bases
-        // used by the reduced read set.
-        final File originalBAMInputFile = new File(TEST_DATA_DIR, "CEUTrio.HiSeq.WGS.b37.NA12878.20.first.8000.bam");
-        final File referenceFile = new File(TEST_DATA_DIR, "human_g1k_v37.20.subset.fasta");
+        final File originalBAMInputFile = new File(TEST_DATA_DIR, testFileName);
+        final File referenceFile = new File(TEST_DATA_DIR, referenceFileName);
 
-        // retrieve all records from the bam and reset the indexing bins to keep comparisons with
-        // cram records from failing
         List<SAMRecord> originalBAMRecords = getSAMRecordsFromFile(originalBAMInputFile, referenceFile);
-        for (int i = 0; i < originalBAMRecords.size(); i++) {
-            originalBAMRecords.get(i).setIndexingBin(null);
-        }
 
         // write the BAM records to a temporary CRAM
-        final File tempCRAMFile = File.createTempFile("testBAMThroughCRAMRoundTrip", CramIO.CRAM_FILE_EXTENSION);
+        final File tempCRAMFile = File.createTempFile("testBAMThroughCRAMRoundTrip", FileExtensions.CRAM);
         tempCRAMFile.deleteOnExit();
         SAMFileHeader samHeader = getFileHeader(originalBAMInputFile, referenceFile);
         writeRecordsToFile(originalBAMRecords, tempCRAMFile, referenceFile, samHeader);
@@ -308,12 +329,12 @@ public class CRAMComplianceTest extends HtsjdkTest {
         List<SAMRecord> cramRecords = getSAMRecordsFromFile(tempCRAMFile, referenceFile);
         Assert.assertEquals(cramRecords.size(), originalBAMRecords.size());
         for (int i = 0; i < originalBAMRecords.size(); i++) {
-            Assert.assertEquals(originalBAMRecords.get(i), cramRecords.get(i));
+            Assert.assertEquals(cramRecords.get(i), originalBAMRecords.get(i));
         }
     }
 
     @Test
-    public void testBAMThroughCRAMRoundTripViaPath() throws IOException, NoSuchAlgorithmException {
+    public void testBAMThroughCRAMRoundTripViaPath() throws IOException {
         final File TEST_DATA_DIR = new File("src/test/resources/htsjdk/samtools/cram");
 
         // These files are reduced versions of the CEUTrio.HiSeq.WGS.b37.NA12878.20.21.bam and human_g1k_v37.20.21.fasta
@@ -324,16 +345,11 @@ public class CRAMComplianceTest extends HtsjdkTest {
         final File originalBAMInputFile = new File(TEST_DATA_DIR, "CEUTrio.HiSeq.WGS.b37.NA12878.20.first.8000.bam");
         final File referenceFile = new File(TEST_DATA_DIR, "human_g1k_v37.20.subset.fasta");
 
-        // retrieve all records from the bam and reset the indexing bins to keep comparisons with
-        // cram records from failing
         List<SAMRecord> originalBAMRecords = getSAMRecordsFromFile(originalBAMInputFile, referenceFile);
-        for (int i = 0; i < originalBAMRecords.size(); i++) {
-            originalBAMRecords.get(i).setIndexingBin(null);
-        }
 
         // write the BAM records to a temporary CRAM
         try (FileSystem jimfs = Jimfs.newFileSystem(Configuration.unix())) {
-            final Path tempCRAM = jimfs.getPath("testBAMThroughCRAMRoundTrip" + CramIO.CRAM_FILE_EXTENSION);
+            final Path tempCRAM = jimfs.getPath("testBAMThroughCRAMRoundTrip" + FileExtensions.CRAM);
             SAMFileHeader samHeader = getFileHeader(originalBAMInputFile, referenceFile);
             writeRecordsToPath(originalBAMRecords, tempCRAM, referenceFile, samHeader);
 
@@ -341,7 +357,7 @@ public class CRAMComplianceTest extends HtsjdkTest {
             List<SAMRecord> cramRecords = getSAMRecordsFromPath(tempCRAM, referenceFile);
             Assert.assertEquals(cramRecords.size(), originalBAMRecords.size());
             for (int i = 0; i < originalBAMRecords.size(); i++) {
-                Assert.assertEquals(originalBAMRecords.get(i), cramRecords.get(i));
+                Assert.assertEquals(cramRecords.get(i), originalBAMRecords.get(i));
             }
         }
     }
