@@ -1,13 +1,12 @@
 package htsjdk.samtools;
 
 import htsjdk.HtsjdkTest;
-import htsjdk.samtools.cram.CRAIIndex;
 import htsjdk.samtools.cram.ref.ReferenceSource;
+import htsjdk.samtools.cram.structure.CRAMEncodingStrategy;
 import htsjdk.samtools.reference.InMemoryReferenceSequenceFile;
 import htsjdk.samtools.seekablestream.ByteArraySeekableStream;
 import htsjdk.samtools.seekablestream.SeekableStream;
 import htsjdk.samtools.util.CloseableIterator;
-import htsjdk.samtools.util.Log;
 import htsjdk.samtools.util.TestUtil;
 import org.testng.Assert;
 import org.testng.annotations.BeforeTest;
@@ -152,9 +151,7 @@ public class CRAMFileWriterWithIndexTest extends HtsjdkTest {
     }
 
     @BeforeTest
-    public void beforeTest() throws Exception {
-        Log.setGlobalLogLevel(Log.LogLevel.ERROR);
-
+    public void beforeTest() {
         header = new SAMFileHeader();
         header.setSortOrder(SAMFileHeader.SortOrder.coordinate);
         SAMReadGroupRecord readGroupRecord = new SAMReadGroupRecord("1");
@@ -176,7 +173,8 @@ public class CRAMFileWriterWithIndexTest extends HtsjdkTest {
         ByteArrayOutputStream indexOS = new ByteArrayOutputStream();
         CRAMFileWriter writer = new CRAMFileWriter(os, indexOS, source, header, null);
 
-        int readPairsPerSequence = CRAMContainerStreamWriter.DEFAULT_RECORDS_PER_SLICE;
+        // use the defaults
+        int readPairsPerSequence = new CRAMEncodingStrategy().getReadsPerSlice();
 
         for (SAMSequenceRecord sequenceRecord : header.getSequenceDictionary().getSequences()) {
             int alignmentStart = 1;
