@@ -2,7 +2,6 @@ package htsjdk.samtools;
 
 import htsjdk.HtsjdkTest;
 import htsjdk.samtools.cram.CRAIEntry;
-import htsjdk.samtools.cram.CRAIIndex;
 import htsjdk.samtools.seekablestream.SeekableFileStream;
 import htsjdk.samtools.seekablestream.SeekableMemoryStream;
 import htsjdk.samtools.seekablestream.SeekableStream;
@@ -16,8 +15,7 @@ import java.io.File;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
-import java.util.ArrayList;
-import java.util.List;
+import java.util.Collections;
 import java.util.zip.GZIPOutputStream;
 
 public class SamIndexesTest extends HtsjdkTest {
@@ -41,6 +39,26 @@ public class SamIndexesTest extends HtsjdkTest {
         }
     }
 
+    @Test
+    public void testEmptyCsi() throws IOException {
+        final File csiFile = File.createTempFile("test", ".csi");
+        csiFile.deleteOnExit();
+        final FileOutputStream fos = new FileOutputStream(csiFile);
+        fos.write(SamIndexes.CSI.magic);
+        fos.close();
+
+
+        final ByteArrayOutputStream baos = new ByteArrayOutputStream();
+        baos.write(SamIndexes.CSI.magic);
+        baos.close();
+
+        final InputStream inputStream = SamIndexes.asBaiStreamOrNull(new ByteArrayInputStream(baos.toByteArray()), null);
+        for (final byte b : SamIndexes.CSI.magic) {
+            Assert.assertEquals(inputStream.read(), 0xFF & b);
+        }
+    }
+
+
     @Test(expectedExceptions = NullPointerException.class)
     public void testCraiRequiresDictionary() throws IOException {
         final ByteArrayOutputStream baos = new ByteArrayOutputStream();
@@ -70,15 +88,8 @@ public class SamIndexesTest extends HtsjdkTest {
 
         SAMFileHeader header = new SAMFileHeader();
         header.setSortOrder(SAMFileHeader.SortOrder.coordinate);
-        CRAMCRAIIndexer indexer = new CRAMCRAIIndexer(baos, header);
-        final CRAIEntry entry = new CRAIEntry();
-        entry.sequenceId = 0;
-        entry.alignmentStart = 1;
-        entry.alignmentSpan = 2;
-        entry.sliceOffset = 3;
-        entry.sliceSize = 4;
-        entry.containerStartOffset = 5;
-        indexer.addEntry(entry);
+        final CRAIEntry entry = new CRAIEntry(0, 1, 2, 5, 3, 4);
+        final CRAMCRAIIndexer indexer = new CRAMCRAIIndexer(baos, header, Collections.singleton(entry));
         indexer.finish();
         baos.close();
 
@@ -91,11 +102,11 @@ public class SamIndexesTest extends HtsjdkTest {
         baos = new ByteArrayOutputStream();
         IOUtil.copyStream(baiStream, baos);
         final CachingBAMFileIndex bamIndex = new CachingBAMFileIndex(new SeekableMemoryStream(baos.toByteArray(), null), dictionary);
-        final BAMFileSpan span = bamIndex.getSpanOverlapping(entry.sequenceId, entry.alignmentStart, entry.alignmentStart);
+        final BAMFileSpan span = bamIndex.getSpanOverlapping(entry.getSequenceId(), entry.getAlignmentStart(), entry.getAlignmentStart());
         Assert.assertNotNull(span);
         final long[] coordinateArray = span.toCoordinateArray();
         Assert.assertEquals(coordinateArray.length, 2);
-        Assert.assertEquals(coordinateArray[0] >> 16, entry.containerStartOffset);
+        Assert.assertEquals(coordinateArray[0] >> 16, entry.getContainerStartByteOffset());
         Assert.assertEquals(coordinateArray[1] & 0xFFFF, 1);
     }
 
@@ -107,15 +118,8 @@ public class SamIndexesTest extends HtsjdkTest {
 
         SAMFileHeader header = new SAMFileHeader();
         header.setSortOrder(SAMFileHeader.SortOrder.coordinate);
-        CRAMCRAIIndexer indexer = new CRAMCRAIIndexer(fos, header);
-        final CRAIEntry entry = new CRAIEntry();
-        entry.sequenceId = 0;
-        entry.alignmentStart = 1;
-        entry.alignmentSpan = 2;
-        entry.sliceOffset = 3;
-        entry.sliceSize = 4;
-        entry.containerStartOffset = 5;
-        indexer.addEntry(entry);
+        final CRAIEntry entry = new CRAIEntry(0, 1, 2, 5, 3, 4);
+        final CRAMCRAIIndexer indexer = new CRAMCRAIIndexer(fos, header, Collections.singleton(entry));
         indexer.finish();
         fos.close();
 
@@ -126,11 +130,11 @@ public class SamIndexesTest extends HtsjdkTest {
         Assert.assertNotNull(baiStream);
 
         final CachingBAMFileIndex bamIndex = new CachingBAMFileIndex(baiStream, dictionary);
-        final BAMFileSpan span = bamIndex.getSpanOverlapping(entry.sequenceId, entry.alignmentStart, entry.alignmentStart);
+        final BAMFileSpan span = bamIndex.getSpanOverlapping(entry.getSequenceId(), entry.getAlignmentStart(), entry.getAlignmentStart());
         Assert.assertNotNull(span);
         final long[] coordinateArray = span.toCoordinateArray();
         Assert.assertEquals(coordinateArray.length, 2);
-        Assert.assertEquals(coordinateArray[0] >> 16, entry.containerStartOffset);
+        Assert.assertEquals(coordinateArray[0] >> 16, entry.getContainerStartByteOffset());
         Assert.assertEquals(coordinateArray[1] & 0xFFFF, 1);
     }
 
@@ -166,15 +170,8 @@ public class SamIndexesTest extends HtsjdkTest {
         final FileOutputStream fos = new FileOutputStream(file);
         SAMFileHeader header = new SAMFileHeader();
         header.setSortOrder(SAMFileHeader.SortOrder.coordinate);
-        CRAMCRAIIndexer indexer = new CRAMCRAIIndexer(fos, header);
-        final CRAIEntry entry = new CRAIEntry();
-        entry.sequenceId = 0;
-        entry.alignmentStart = 1;
-        entry.alignmentSpan = 2;
-        entry.sliceOffset = 3;
-        entry.sliceSize = 4;
-        entry.containerStartOffset = 5;
-        indexer.addEntry(entry);
+        final CRAIEntry entry = new CRAIEntry(0, 1, 2, 5, 3, 4);
+        final CRAMCRAIIndexer indexer = new CRAMCRAIIndexer(fos, header, Collections.singleton(entry));
         indexer.finish();
         fos.close();
 
@@ -184,11 +181,11 @@ public class SamIndexesTest extends HtsjdkTest {
         final ByteArrayOutputStream baos = new ByteArrayOutputStream();
         IOUtil.copyStream(baiStream, baos);
         final CachingBAMFileIndex bamIndex = new CachingBAMFileIndex(new SeekableMemoryStream(baos.toByteArray(), null), dictionary);
-        final BAMFileSpan span = bamIndex.getSpanOverlapping(entry.sequenceId, entry.alignmentStart, entry.alignmentStart);
+        final BAMFileSpan span = bamIndex.getSpanOverlapping(entry.getSequenceId(), entry.getAlignmentStart(), entry.getAlignmentStart());
         Assert.assertNotNull(span);
         final long[] coordinateArray = span.toCoordinateArray();
         Assert.assertEquals(coordinateArray.length, 2);
-        Assert.assertEquals(coordinateArray[0] >> 16, entry.containerStartOffset);
+        Assert.assertEquals(coordinateArray[0] >> 16, entry.getContainerStartByteOffset());
         Assert.assertEquals(coordinateArray[1] & 0xFFFF, 1);
     }
 }
