@@ -22,6 +22,7 @@ import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.net.MalformedURLException;
+import java.net.URI;
 import java.net.URL;
 import java.nio.ByteBuffer;
 import java.nio.channels.SeekableByteChannel;
@@ -37,7 +38,6 @@ import java.util.zip.Inflater;
 
 public class SamReaderFactoryTest extends HtsjdkTest {
     private static final File TEST_DATA_DIR = new File("src/test/resources/htsjdk/samtools");
-
     private static final Log LOG = Log.getInstance(SamReaderFactoryTest.class);
 
     @Test(dataProvider = "variousFormatReaderTestCases")
@@ -250,7 +250,7 @@ public class SamReaderFactoryTest extends HtsjdkTest {
 
             sources.add(new SamInputResource(composeInputResourceForType(dataType, false)));
             for (final InputResource.Type indexType : InputResource.Type.values()) {
-                if (indexType.equals(InputResource.Type.SRA_ACCESSION))
+                if (indexType.equals(InputResource.Type.SRA_ACCESSION) || indexType.equals(InputResource.Type.HTSGET))
                     continue;
 
                 sources.add(new SamInputResource(
@@ -285,6 +285,9 @@ public class SamReaderFactoryTest extends HtsjdkTest {
                 } catch (final FileNotFoundException e) {
                     throw new RuntimeIOException(e);
                 }
+            case HTSGET:
+                return new HtsgetInputResource(URI.create(
+                    HtsgetBAMFileReaderTest.HTSGET_ENDPOINT + HtsgetBAMFileReaderTest.LOCAL_PREFIX + f.getName()));
             default:
                 throw new IllegalStateException();
         }
@@ -330,6 +333,30 @@ public class SamReaderFactoryTest extends HtsjdkTest {
         }
     }
 
+    @DataProvider(name = "URIFallbackProvider")
+    public Object[][] URIFallbackProvider() throws MalformedURLException {
+        return new Object[][]{
+            {
+                URI.create("htsget://127.0.0.1:3000/reads/htsjdk_test.index_test.bam"),
+                InputResource.Type.HTSGET,
+            },
+            {
+                localBam.toURI(),
+                InputResource.Type.PATH,
+            },
+            {
+                URI.create("http://test.url"),
+                InputResource.Type.URL,
+            },
+        };
+    }
+
+    @Test(dataProvider = "URIFallbackProvider")
+    public void testOpenURIFallback(final URI uri, final InputResource.Type type) {
+        final SamInputResource resource = SamInputResource.of(uri);
+        Assert.assertEquals(resource.data().type(), type);
+    }
+
     final Set<List<SAMRecord>> observedRecordOrdering1 = new HashSet<>();
     final Set<List<SAMRecord>> observedRecordOrdering3 = new HashSet<>();
     final Set<List<SAMRecord>> observedRecordOrdering20 = new HashSet<>();
