@@ -28,13 +28,18 @@ package htsjdk.variant;
 import htsjdk.HtsjdkTest;
 import htsjdk.samtools.SAMSequenceDictionary;
 import htsjdk.samtools.SAMSequenceRecord;
+import htsjdk.samtools.util.Tuple;
+import htsjdk.utils.ValidationUtils;
 import htsjdk.variant.variantcontext.Genotype;
 import htsjdk.variant.variantcontext.VariantContext;
 import htsjdk.variant.vcf.VCFConstants;
+import htsjdk.variant.vcf.VCFFileReader;
+import htsjdk.variant.vcf.VCFHeader;
 import org.testng.Assert;
 
 import java.io.File;
 import java.io.IOException;
+import java.nio.file.Path;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashSet;
@@ -124,6 +129,23 @@ public class VariantBaseTest extends HtsjdkTest {
         return new SAMSequenceDictionary(contigs);
     }
 
+    /**
+     * Reads an entire VCF into memory, returning both its VCFHeader and all VariantContext records in
+     * the vcf.
+     *
+     * For unit/integration testing purposes only! Do not call this method from actual tools!
+     *
+     * @param vcfPath Path of file to be loaded
+     * @return A Tuple with the VCFHeader as the first element, and a List of all VariantContexts from the VCF
+     *         as the second element
+     */
+    public static Tuple<VCFHeader, List<VariantContext>> readEntireVCFIntoMemory(final Path vcfPath) {
+        ValidationUtils.nonNull(vcfPath);
+        try ( final VCFFileReader vcfReader = new VCFFileReader(vcfPath, false) ){
+            return new Tuple<>(vcfReader.getFileHeader(), vcfReader.iterator().toList());
+        }
+    }
+
     /**
      * Asserts that the two provided VariantContext objects are equal.
      *
